package de.mdelab.instanceGraphEditor.ui.editor;

import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;

import de.mdelab.instanceGraphEditor.ui.actions.ExpandNodeAction;
import de.mdelab.instanceGraphEditor.ui.actions.HideChildrenAction;
import de.mdelab.instanceGraphEditor.ui.actions.HideNodeAction;
import de.mdelab.instanceGraphEditor.ui.actions.PrintOutNodeAction;
import de.mdelab.instanceGraphEditor.ui.model.Graph;
import de.mdelab.instanceGraphEditor.ui.model.InstanceGraphFactory;
import de.mdelab.instanceGraphEditor.ui.model.Node;
import de.mdelab.instanceGraphEditor.ui.parts.GraphEditPartFactory;

public class InstanceGraphEditor extends GraphicalEditorWithFlyoutPalette {

	private Resource sdResource;
	
	private Graph graph;
	
	private EObject root;
	
	public InstanceGraphEditor() {
		setEditDomain(new DefaultEditDomain(this));
	}
	
	@Override protected void initializeGraphicalViewer() {
		super.initializeGraphicalViewer();
		getGraphicalViewer().setContents(graph);
	}	
	
	@Override protected void configureGraphicalViewer() {
		super.configureGraphicalViewer();
		getGraphicalViewer().setEditPartFactory(new GraphEditPartFactory());
        getGraphicalViewer().setContextMenu(new GraphicalEditorContextMenuProvider(getGraphicalViewer(), getActionRegistry()));
	}	
	
	@Override protected PaletteRoot getPaletteRoot() {
		return new GraphicalEditorPalette();
	}
	
	public Graph getGraph(){
		return graph;
	}
	
    @Override
    protected void createActions() {
    	IAction action = new ExpandNodeAction(this);
        getActionRegistry().registerAction(action);
        getSelectionActions().add(action.getId());
    	
        action = new HideChildrenAction(this);
        getActionRegistry().registerAction(action);
        getSelectionActions().add(action.getId());
        
        action = new HideNodeAction(this);
        getActionRegistry().registerAction(action);
        getSelectionActions().add(action.getId());
        
        action = new PrintOutNodeAction(this);
        getActionRegistry().registerAction(action);
        getSelectionActions().add(action.getId());
        
        super.createActions();
    }

	@Override 
	public void doSave(IProgressMonitor monitor) {
		if(sdResource == null) {
			return;
		}
		
		try {
			sdResource.save(null);
		} catch(IOException e) {
			e.printStackTrace();
			sdResource = null;
		}
	}

	@Override 
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		super.init(site, input);
		loadInput(input);		
	}
	
	private void loadInput(IEditorInput input) {
		
		graph = InstanceGraphFactory.INSTANCE.getGraph();
		Node firstNode = null;
		if (input instanceof FileEditorInput) {
			IFileEditorInput fileInput = (IFileEditorInput) input;
			IFile file = fileInput.getFile();
			ResourceSet resourceSet = new ResourceSetImpl();
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());

			URI fileUri = URI.createFileURI(file.getLocation().toString());
			Resource resource = resourceSet.getResource(fileUri, true);
			root = (EObject) resource.getContents().get(0);
			firstNode = InstanceGraphFactory.INSTANCE.createNode(graph, root, null);
			
		}
		else if (input instanceof InstanceGraphEditorInput){
			
			root = ((Resource)((InstanceGraphEditorInput)input).getObject().eResource()).getContents().get(0);
			EObject firstObject = ((InstanceGraphEditorInput)input).getObject();
			firstNode = InstanceGraphFactory.INSTANCE.createNode(graph, firstObject, null);
		}
		
		graph.addNode(firstNode);
	}
}
