package de.mdelab.instanceGraphEditor.ui.parts;

import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.graph.DirectedGraph;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.jface.viewers.TextCellEditor;

import de.mdelab.instanceGraphEditor.ui.figures.AttributesFigure;
import de.mdelab.instanceGraphEditor.ui.figures.EditableLabel;
import de.mdelab.instanceGraphEditor.ui.figures.NodeFigure;
import de.mdelab.instanceGraphEditor.ui.figures.SelfLinkAnchor.AnchorType;
import de.mdelab.instanceGraphEditor.ui.model.Node;
import de.mdelab.instanceGraphEditor.ui.policies.NodeAttibuteValueDirectEditPolicy;
import de.mdelab.instanceGraphEditor.ui.policies.NodeGraphicalNodeEditPolicy;
import de.mdelab.instanceGraphEditor.ui.policies.NodesEditPolicy;

public class NodesEditPart extends AbstractGraphicalEditPart implements	Observer, NodeEditPart {
	
	
	public NodesEditPart(Node node) {
		setModel(node);
	}
	
	protected IFigure createFigure() {
		return new NodeFigure(new Label(((Node)getModel()).getName()));
	}
	
	public void contributeNodesToGraph(DirectedGraph graph,	Map map) {
		
		GraphAnimation.recordInitialState(getContentPane());
		
		org.eclipse.draw2d.graph.Node n = new org.eclipse.draw2d.graph.Node(this);
		n.width = getFigure().getPreferredSize().width;
		n.height = getFigure().getPreferredSize().height;
		n.setPadding(new Insets(30, 30, 30, 30));
		map.put(this, n);
		graph.nodes.add(n);
	}
	
	public void contributeEdgesToGraph(DirectedGraph graph, Map map) {
		
		List outgoing = getSourceConnections();
		for (int i = 0; i < outgoing.size(); i++) {
			if( ((LinkEditPart) getSourceConnections().get(i)).getSource() != ((LinkEditPart) getSourceConnections().get(i)).getTarget()){
			LinkEditPart part = (LinkEditPart) getSourceConnections().get(i);
			part.contributeToGraph(graph, map);
			}
		}
	}
	
	protected void applyGraphResults(DirectedGraph graph, Map map) {
		org.eclipse.draw2d.graph.Node n = (org.eclipse.draw2d.graph.Node) map.get(this);
		getFigure().setBounds(new Rectangle(n.x, n.y, n.width, getFigure().getPreferredSize().height));
	}

	protected void createEditPolicies() {
        installEditPolicy(EditPolicy.COMPONENT_ROLE, new NodesEditPolicy());
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new NodeGraphicalNodeEditPolicy());
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new NodeAttibuteValueDirectEditPolicy());
	}
	
	@Override
	public void performRequest(Request req) {
		if (req.getType() == RequestConstants.REQ_DIRECT_EDIT) {
			performDirectEditing(((DirectEditRequest)req).getLocation());
		}
	}

	private void performDirectEditing(Point location) {
		
		if( ((NodeFigure)getFigure()).findFigureAt(location) instanceof Label	){
			Label attr = (Label)((NodeFigure)getFigure()).findFigureAt(location);
			((NodeFigure)getFigure()).setSelectedAttribute(attr);
			NodeAttributeDirectEditManager manager = new NodeAttributeDirectEditManager(this, TextCellEditor.class, new NodeCellEditorLocator(attr), attr);
			manager.show();
		}
	}

	public void refreshVisuals() {
		NodeFigure figure = (NodeFigure) getFigure();
		Node node = (Node) getModel();
		GraphEditPart parent = (GraphEditPart) getParent();
		figure.getNameLabel().setText(node.getName());
		figure.getAttributesFigure().removeAll();			
		for(EAttribute attr : ((Node)getModel()).getAttributes()){
			figure.getAttributesFigure().add(new Label(attr.getName() + " = " +  ((Node)getModel()).getAttributeValue(attr) ));
		}
		Rectangle r = new Rectangle(node.getConstraint());
		parent.setLayoutConstraint(this, figure, r);
	}

	public void activate() {
		if (!isActive())
			((Node) getModel()).addObserver(this);
		super.activate();
	}

	public void deactivate() {
		if (isActive())
			((Node) getModel()).deleteObserver(this);
		super.deactivate();
	}

	public void update(Observable observable, Object message) {

		refreshVisuals();
		refreshSourceConnections();
		refreshTargetConnections();
	}

	protected List getModelSourceConnections() {
		return ((Node) getModel()).getSourceLinks();
	}

	protected List getModelTargetConnections() {
		return ((Node) getModel()).getTargetConnections();
	}

	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		if(connection.getSource() == connection.getTarget())
			return ((NodeFigure) getFigure()).getSelfConnectionAnchor(AnchorType.SOURCE_ANCHOR);
		return ((NodeFigure) getFigure()).getConnectionAnchor();
	}

	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		if(connection.getSource() == connection.getTarget())
			return ((NodeFigure) getFigure()).getSelfConnectionAnchor(AnchorType.TARGET_ANCHOR);
		return ((NodeFigure) getFigure()).getConnectionAnchor();
	}

	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return ((NodeFigure) getFigure()).getConnectionAnchor();
	}

	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return ((NodeFigure) getFigure()).getConnectionAnchor();
	}
}