package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.actparameter;

import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Composite;

import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.ITableViewerListener;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.TableViewerWrapper;

/**
 * Table to edit story diagram activity parameters for Eclipse launch configuration
 */
public abstract class SDActivityParameterTableViewer<Activity, ActivityParameter extends SDTableActivityParameter<?, Classifier>, Classifier> extends TableViewerWrapper<Activity, ActivityParameter, TableActivityParameterColumnType> {
	
	private SDInstanceObjectEditingSupport<ActivityParameter, Classifier> instObjEditingSupport;

	public SDActivityParameterTableViewer( final Composite p_parent ) {
		super( p_parent, false );
	}
	
	public void initData( 	final Activity p_root,
							final Set<Resource> p_instanceObjectRes,
							final Map<String, String> p_rowData ) {
		instObjEditingSupport.setInstObjectsResources( p_instanceObjectRes );
		tableViewer.setInput( getTableData().getRowObjects() );
	}
	
	public void initData( 	final Activity p_activity,
							final Set<Resource> p_instanceObjectRes ) {
		instObjEditingSupport.setInstObjectsResources( p_instanceObjectRes );
		//tableData.initData( p_activity );
		tableViewer.setInput( getTableData().getRowObjects() );
	}
	
	@Override
	public void addViewerListener( final ITableViewerListener<ActivityParameter> p_listener ) {
		instObjEditingSupport.addEditorListener( p_listener );
		
		super.addViewerListener( p_listener );
	}
	
	@Override
	public boolean removeViewerListener( final ITableViewerListener<ActivityParameter> p_listener ) {
		return instObjEditingSupport.removeEditorListener( p_listener ) || super.removeViewerListener( p_listener );
	}
	
	protected void createColumns() {
		createViewerColumn( 100, "Name", new ColumnLabelProvider() {
			
			@Override
			public String getText( final Object p_element ) {
			    final SDTableActivityParameter<?, ?> parameter = (SDTableActivityParameter<?, ?>) p_element;
			    
			    return parameter.getName();
			}
		}, null, TableActivityParameterColumnType.NAME );

		createViewerColumn( 200, "Type", new ColumnLabelProvider() {
			
			@Override
			public String getText( final Object p_element ) {
			    final SDTableActivityParameter<?, ?> parameter = (SDTableActivityParameter<?, ?>) p_element;
			    
			    return labelProvider.getText( parameter.getType() );
			}
		}, null, TableActivityParameterColumnType.TYPE );

		instObjEditingSupport = createInstanceObjectsEditingSupport( tableViewer, labelProvider );
		
		createViewerColumn( 300, "Instance Object", new ColumnLabelProvider() {
			
			@Override
			public String getText( final Object p_element ) {
			    final SDTableActivityParameter<?, ?> parameter = (SDTableActivityParameter<?, ?>) p_element;
			    
			    return labelProvider.getText( parameter.getInstanceObject() );
			}
		}, instObjEditingSupport, TableActivityParameterColumnType.INSTANCE_OBJECT );
	}
	
	@Override
	protected SDTableActivityParameterComparator createTableViewerComparator( final ILabelProvider p_labelProvider ) {
		return new SDTableActivityParameterComparator( p_labelProvider );
	}
	
	protected abstract SDInstanceObjectEditingSupport<ActivityParameter, Classifier> createInstanceObjectsEditingSupport( 	TableViewer p_tableViewer, 
																															ILabelProvider p_labelProvider );
	
	public Map<String, String> getDataString() {
		return getTableData().getDataString();
	}
	
	@Override
	public SDTableActivityParameterData<Activity, ActivityParameter> getTableData() {
		return (SDTableActivityParameterData<Activity, ActivityParameter>) super.getTableData();
	}
}